# -*- coding: utf-8 -*-
from PyQt4.QtCore import Qt, QVariant, QObject, QPoint, QRect, SIGNAL
from PyQt4.QtGui import *

from utilidades import Utilidades

class DialogoMedirDistancia( QDialog ):
    """ Dilogo Medir Distancia """
    def __init__( self, parent=None ):
        QDialog.__init__( self, parent )

        self.unidades = "grados"  # Se actualiza con la 1era capa cargada al mapa

        # Interfaz
        self.setWindowTitle( 'Medir distancias' )

        frame = QFrame( self )
        frame.setGeometry( QRect( 10,10,193,123 ) )
        frame.setFrameShape( QFrame.StyledPanel )
        frame.setFrameShadow( QFrame.Sunken )
        frame.setObjectName( "frame" )

        self.listWidget = QListWidget( frame )
        self.listWidget.setGeometry( QRect( 0,31,192,91 ) )

        font = QFont()
        font.setFamily( "Arial" )
        font.setPointSize( 10 )
        self.listWidget.setFont( font )
        self.listWidget.setMouseTracking( True )
        self.listWidget.setLayoutDirection( Qt.RightToLeft )
        self.listWidget.setAutoFillBackground( False )
        self.listWidget.setHorizontalScrollBarPolicy( Qt.ScrollBarAlwaysOff )
        self.listWidget.setAlternatingRowColors( True )
        self.listWidget.setSelectionMode( QAbstractItemView.SingleSelection )
        self.listWidget.setSelectionBehavior( QAbstractItemView.SelectRows )
        self.listWidget.setProperty( "isWrapping",QVariant( False ) )
        self.listWidget.setResizeMode( QListView.Fixed )
        self.listWidget.setLayoutMode( QListView.SinglePass )
        self.listWidget.setViewMode( QListView.ListMode )
        self.listWidget.setUniformItemSizes( True )
        self.listWidget.setWordWrap( True )
        self.listWidget.setSelectionRectVisible( False )
        self.listWidget.setObjectName( "listWidget" )

        font = QFont()
        font.setFamily( "Arial" )
        font.setPointSize( 10.5 )
        self.etiquetaUnidades = QLabel( frame )
        self.etiquetaUnidades.setFont( font )
        self.etiquetaUnidades.setGeometry( QRect( 2,2,189,31 ) )
        self.etiquetaUnidades.setFrameShape( QFrame.StyledPanel )
        self.etiquetaUnidades.setFrameShadow( QFrame.Raised )
        self.etiquetaUnidades.setObjectName( "label" )
        self.etiquetaUnidades.setText( "  Segmentos (Grados)" )

        self.txtTotalMedida = QLineEdit( self )
        self.txtTotalMedida.setEnabled( True )
        self.txtTotalMedida.setGeometry( QRect( 50,150,151,23 ) )

        font = QFont()
        font.setFamily( "Arial" )
        font.setPointSize( 10 )
        font.setWeight( 75 )
        font.setBold( True )
        self.txtTotalMedida.setFont( font )
        self.txtTotalMedida.setAcceptDrops( True )
        self.txtTotalMedida.setLayoutDirection( Qt.LeftToRight )
        self.txtTotalMedida.setEchoMode( QLineEdit.Normal )
        self.txtTotalMedida.setAlignment( Qt.AlignRight | Qt.AlignTrailing | Qt.AlignVCenter )
        self.txtTotalMedida.setReadOnly( True )
        self.txtTotalMedida.setObjectName( "txtTotalMedida" )

        label_2 = QLabel( self )
        label_2.setGeometry( QRect( 10,147,41,31 ) )
        label_2.setObjectName( "label_2" )
        label_2.setText( "Total:" )

        btnButtonBox = QDialogButtonBox( self )
        btnButtonBox.setGeometry( QRect( 72,188,70,25 ) )
        btnButtonBox.addButton( 'Cerrar', QDialogButtonBox.AcceptRole )
        btnButtonBox.setObjectName( "btnButtonBox" )

        QObject.connect( btnButtonBox, SIGNAL( "accepted()" ), self.cerrar )

        grid = QGridLayout()
        grid.setSpacing( 0 )

        grid.addWidget( frame, 1, 0, 1, 2, Qt.AlignCenter )
        grid.addWidget( label_2, 1, 1 )
        grid.addWidget( self.txtTotalMedida, 2, 1 )

        grid.setRowStretch( 3,1 )
        grid.addWidget( btnButtonBox,4,0,1,2, Qt.AlignCenter )

        self.setFixedSize( 213, 226 )

        # Cerrar el dilogo cuando se cierre el visor (Necesario por ser no modal)
        QObject.connect( parent, SIGNAL( "cerrandoVisor()"), self.cerrar )
        QObject.connect( parent, SIGNAL( "unidadesMapaCambiadas" ),
            self.actualizarUnidades )

        # Posicin
        self.psize = parent.geometry()
        self.move( self.psize.width() / 20, self.psize.height() / 1.5 ) # Posicin inicial
        self.posicion = None # Recuerda la posicin del dilogo

        self.utils = Utilidades.obtenerInstancia() # Obtener instancia de Utilidades

        self.medidaTotalActual = 0 # Almacena la medida Total en cada click
        self.inicializar() # Inicializa las medidas

    def actualizarUnidades( self, units ):
        """ SLOT para actualizar las unidades en las etiquetas"""
        if units == 0:
            self.unidades = "m."
            self.etiquetaUnidades.setText( "  Segmentos (Metros)" )
        elif units == 1:
            self.unidades = "pies"
            self.etiquetaUnidades.setText( "  Segmentos (Pies)" )
        elif units == 2:
            self.unidades = "grados"
            self.etiquetaUnidades.setText( "  Segmentos (Grados)" )
        elif units == 3:
            self.unidades = "desconocida"
            self.etiquetaUnidades.setText( "  Segmentos" )

        self.txtTotalMedida.setText( "0,00" + " " + self.unidades )

    def agregarMedidaSegmento( self ):
        """ Agrega un nuevo Item 0 al List de segmentos y suma al Total"""
        item = QListWidgetItem( self.listWidget )
        item.setText( self.utils.formatearNumero( 0,2 ) )

        self.listWidget.scrollToItem( item )

        # Actualizar la variable medidaTotalActual con el total actual
        num = len( self.unidades )
        self.medidaTotalActual = float( self.txtTotalMedida.text().replace( ".","" )[ :-num ].replace( ",","." ) )

    def modificarMedidaSegmento( self, medida ):
        """ Modifica el valor del ltimo item de la lista segmentos """
        item = self.listWidget.item( self.listWidget.count() - 1 )
        item.setText( self.utils.formatearNumero( medida, 2 ) )

        self.actualizarMedidaTotal( medida )

    def actualizarMedidaTotal( self, medida ):
        """ Suma el ltimo valor del Total con el valor dinmico del segmento """
        medida += self.medidaTotalActual
        self.txtTotalMedida.setText( self.utils.formatearNumero( medida, 2 ) + " " + self.unidades )

    def inicializar( self ):
        self.listWidget.clear()
        self.txtTotalMedida.setText( "0,00" + " " + self.unidades )
        self.medidaTotalActual = 0

    def closeEvent( self, event ):
        self.posicion = QPoint( self.x(), self.y() ) # Recordar la posicin al cerrar

    def mostrar( self ):
        if self.posicion:
            self.move( self.posicion.x(), self.posicion.y() ) # Posiciona el dilogo
	    self.posicion = None
	if not self.isVisible():
	    self.show()

    def cerrar( self ):
        self.close()